import exec_helpers
import yaml

from exec_helpers import CalledProcessError
from retry import retry
from si_tests import logger
from si_tests.utils import utils

LOG = logger.logger


class Helm3Manager(object):
    binary = utils.get_binary_path("helm")

    def __init__(self, namespace='openstack'):
        self.namespace = namespace

    @retry(CalledProcessError, delay=60, tries=10, logger=LOG)
    def run_cmd(self, cmd):
        cmd = " ".join([self.binary, *cmd])
        res = exec_helpers.Subprocess().execute(cmd)
        if res.exit_code:
            raise CalledProcessError(result=res, expected=(0,))
        return (res.stdout_str, res.stderr_str)

    def list(self, args=None):
        args = args or []
        cmd = [
            "list",
            "--namespace",
            self.namespace,
            "-o",
            "json",
            *args,
        ]
        stdout, stderr = self.run_cmd(cmd)
        if isinstance(res := yaml.safe_load(stdout), list):
            return res
        else:
            LOG.error("Helm return unexpected answer " + stdout)
            return []

    def get_release_values(self, name, args=None):
        args = args or []
        cmd = [
            "get",
            "values",
            "--namespace",
            self.namespace,
            name,
            "-o",
            "json",
            *args,
        ]
        stdout, stderr = self.run_cmd(cmd)
        return yaml.safe_load(stdout)

    def get_releases_values(self, args=None):
        args = args or []
        res = {}
        for release in self.list(args):
            name = release['name']
            res[name] = self.get_release_values(name)
        return res
