import exec_helpers
import yaml
import json
import pytest

from si_tests import settings
from si_tests import logger
import si_tests.utils.templates as utils
from si_tests.utils import waiters

LOG = logger.logger


@pytest.mark.parametrize("_", [f"CLUSTER_NAME={settings.TARGET_CLUSTER}"])
def test_tungsten_pytest(kaas_manager, _):
    result_pod_path = "/tungsten-pytest/data"
    result_pod_file = "results.xml"
    test_pod_name = "tf-test-tungsten-pytest"
    test_pod_ns = "tf"
    test_report_name = "tf_report"
    operator_name = 'openstack-tf'
    cluster_name = settings.TARGET_CLUSTER
    target_namespace = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    target_cluster = target_namespace.get_cluster(cluster_name)
    kubectl_client = target_cluster.k8sclient

    # Check and cleanup previous run
    tfoperator = kubectl_client.tfoperator.get(name=operator_name, namespace=test_pod_ns) or {}
    controllers = tfoperator.data.get('spec', {}).get('controllers', {})
    controllers = dict(filter(lambda x: not x[0] == "tf-test", controllers.items()))
    LOG.info("Cleanup previous run if it present!")
    tfoperator.patch({"spec": {"controllers": controllers}})
    pod_present = kubectl_client.pods.present(test_pod_name, test_pod_ns)
    if pod_present:
        LOG.info("Delete test pod")
        kubectl_client.pods.api.delete_namespaced_pod(test_pod_name, test_pod_ns)
    pod_result_present = kubectl_client.pods.present(test_pod_name+"-result", test_pod_ns)
    if pod_result_present:
        LOG.info("Delete result pod")
        kubectl_client.pods.api.delete_namespaced_pod(test_pod_name+"-result", test_pod_ns)

    # Run tungsten pytest
    tfoperator.patch({"spec":
                      {"controllers":
                       {"tf-test":
                        {"tungsten-pytest":
                         {"enabled": True}}}}})
    LOG.info("Wait for pod to be present...")
    waiters.wait(lambda: len([p for p in kubectl_client.pods.list(namespace=test_pod_ns,
                                                                  name_prefix=test_pod_name)]) > 0,
                 timeout=120, interval=10,  timeout_msg=f"Timeout wait for pod {test_pod_name}")
    test_pod = kubectl_client.pods.get(test_pod_name, test_pod_ns)
    LOG.info("Wait for pod to be Running...")
    test_pod.wait_phase('Running')
    LOG.info("Wait for pod to be Completed...")
    test_pod.wait_phase('Succeeded')
    logs = kubectl_client.pods.api.read_namespaced_pod_log(
        name=test_pod_name, namespace=test_pod_ns)
    LOG.info(logs)

    # Disable tungsten pytest
    tfoperator.patch({"spec":
                      {"controllers":
                       {"tf-test":
                        {"tungsten-pytest":
                         {"enabled": False}}}}})
    pod_present = kubectl_client.pods.present(test_pod_name, test_pod_ns)
    if pod_present:
        LOG.info("Delete test pod")
        kubectl_client.pods.api.delete_namespaced_pod(test_pod_name,
                                                      test_pod_ns)

    # Run result pod for extracting results.xml from PV
    options = {
        'RESULT_POD_NAME': test_pod_name+"-result",
        'RESULT_POD_NS': test_pod_ns,
        'RESULT_POD_PV': "tf-test-data-volume",
        'RESULT_POD_PVC': "tf-test-tungsten-pytest",
        'RESULT_POD_MOUNT_PATH': result_pod_path
    }
    templates = utils.render_template(settings.TUNGSTEN_RESULT_POD_YAML, options)
    LOG.debug(templates)
    json_body = json.dumps(yaml.load(templates, Loader=yaml.SafeLoader))
    result_pod = kubectl_client.pods.create(
        name=test_pod_name+"-result",
        namespace=test_pod_ns,
        body=json.loads(json_body))
    result_pod.wait_phase('Running')
    result_pod.wait_test(filepath=result_pod_path+'/'+result_pod_file, timeout=60)
    result_pod.cp_from_pod(source_dir=result_pod_path, source_file=result_pod_file)
    exec_helpers.Subprocess().check_call(
        f"test -f artifacts/results.xml && mv artifacts/results.xml {settings.ARTIFACTS_DIR}/{test_report_name}.xml")
    result_pod.delete()
