import pytest
from si_tests import logger
from si_tests import settings
LOG = logger.logger


@pytest.mark.parametrize("_", [f"CLUSTER_NAME={settings.TARGET_CLUSTER}"])
def test_graceful_reboot(kaas_manager, _, show_step):
    """ Graceful reboot of all machines of the cluster
    Scenario:
        1. Check init state cluster
        2. Create Graceful Reboot Request
        3. Check all machines are rebooted
        4. Check return cluster to init state
    """
    cluster_name = settings.TARGET_CLUSTER
    namespace_name = settings.TARGET_NAMESPACE
    ns = kaas_manager.get_namespace(namespace_name)
    cluster = ns.get_cluster(cluster_name)

    show_step(1)
    LOG.info(f"Check init state on the {cluster._cluster_type} cluster {cluster.namespace}/{cluster.name}")
    cluster.check.check_machines_status()
    cluster.check.check_cluster_readiness()
    cluster.check.check_k8s_nodes()
    cluster.check.wait_graceful_reboot_request(expected_status=False)

    show_step(2)
    LOG.info(f"Creating Graceful Reboot Request for all machines in cluster {cluster.namespace}/{cluster.name}")
    ns.create_gracefulrebootrequest_object(cluster.name, namespace_name)
    cluster.check.wait_graceful_reboot_request(expected_status=True)

    show_step(3)
    LOG.info('Waiting for all machines to reboot')
    boot_time_dict = cluster.get_boot_time_dict(exclude_bastion=True)
    machines_number = len(boot_time_dict.keys())
    # Rebooting BM machines takes about 6-9 minutes
    machines_reboot_timeout = 600 * machines_number
    cluster.check.wait_machines_reboot(boot_time_dict, timeout=machines_reboot_timeout)

    show_step(4)
    LOG.info(f"Check cluster {cluster.namespace}/{cluster.name} for init state")
    cluster.check.wait_graceful_reboot_request(expected_status=False, timeout=600)
    cluster.check.check_machines_status()
    cluster.check.check_cluster_readiness()
