from si_tests.deployments import deploy_refapp
from si_tests.lcm import openstack_lcm
from si_tests import settings


def test_openstack_upgrade_simple(os_manager, openstack_client_manager):
    """Upgrade OS version.
       Parameters required for test execution:
         - KUBECONFIG
         - OPENSTACK_UPGRADE_VERSION
    """

    openstack_lcm.upgrade_openstack(os_manager, openstack_client_manager)


def test_openstack_upgrade_with_refapp(os_manager,
                                       openstack_client_manager):
    """Update OS Controller with refapp tests.
    1) Deploy RefApp
    2) Check RefApp workability before update
    3) Upgrade OpenStack to the new version
    4) Check RefApp workability after update
    5) Delete RefApp

       Parameters required for test execution:
         - KUBECONFIG
         - OSH_OPERATOR_UPDATE_REPO
         - OSH_OPERATOR_UPDATE_TAG
         - OSH_OPERATOR_CHART_URL
         - OSH_OPERATOR_CHART_VERSION
    """

    try:
        deploy_refapp.deploy_refapp(openstack_client_manager)
        openstack_lcm.check_refapp(openstack_client_manager)
        openstack_lcm.upgrade_openstack(os_manager, openstack_client_manager)
        openstack_lcm.check_refapp(openstack_client_manager)
    except Exception:
        if (settings.OPENSTACK_COLLECT_REFAPP_MODE.lower() == "on_error"):
            deploy_refapp.collect_logs_refapp(openstack_client_manager)
        raise
    finally:
        if (settings.OPENSTACK_COLLECT_REFAPP_MODE.lower() == "always"):
            deploy_refapp.collect_logs_refapp(openstack_client_manager)
        deploy_refapp.delete_refapp(openstack_client_manager)
