import yaml

from si_tests import logger
from si_tests import settings
from si_tests.managers.kaas_manager import Manager
from si_tests.utils import templates, utils


LOG = logger.logger


def test_openstack_rally(kaas_manager: Manager):
    kubectl_client = kaas_manager.api

    ns = settings.RALLY_OPENSTACK_NAMESPACE
    configmap_name = settings.RALLY_OPENSTACK_CONFIGMAP_NAME
    pod_name = settings.RALLY_OPENSTACK_POD_NAME

    image = settings.RALLY_OPENSTACK_IMAGE
    scenarios_repo = settings.RALLY_OPENSTACK_SCENARIOS_REPO
    scenarios_branch = settings.RALLY_OPENSTACK_SCENARIOS_BRANCH
    enable_scenarios = settings.RALLY_OPENSTACK_ENABLE_SCENARIOS
    skip_scenarios = settings.RALLY_OPENSTACK_SKIP_SCENARIOS
    task_args = settings.RALLY_OPENSTACK_RALLY_TASK_ARGS

    timeout = settings.RALLY_OPENSTACK_WAIT_TIMEOUT

    LOG.info("Create ConfigMap {}".format(configmap_name))
    config_map_config = yaml.load(
        templates.render_template(
            settings.RALLY_OPENSTACK_CONFIGMAP_YAML,
            {
                'RALLY_OPENSTACK_CONFIGMAP_NAME': configmap_name
            }
        ),
        Loader=yaml.SafeLoader
    )
    if enable_scenarios:
        config_map_config['data']['enable-scenarios'] = enable_scenarios

    if skip_scenarios:
        config_map_config['data']['skip-scenarios'] = skip_scenarios

    if task_args:
        d = yaml.load(task_args, Loader=yaml.SafeLoader)
        s = yaml.load(config_map_config['data']['task-params.yaml'], Loader=yaml.SafeLoader)
        utils.merge_dicts(s, d)
        config_map_config['data']['task-params.yaml'] = yaml.dump(s)

    if kubectl_client.configmaps.present(name=configmap_name, namespace=ns):
        LOG.warning("ConfigMap '{0}' already exists. Deleting..".format(configmap_name))
        kubectl_client.api_core.delete_namespaced_config_map(configmap_name, ns)

    kubectl_client.configmaps.create(name=configmap_name, namespace=ns, body=config_map_config)

    LOG.info("Create Pod {}".format(configmap_name))
    pod_config = yaml.load(
        templates.render_template(
            settings.RALLY_OPENSTACK_POD_YAML,
            {
                'RALLY_OPENSTACK_POD_NAME': pod_name,
                'RALLY_OPENSTACK_IMAGE': image,
                'RALLY_SCENARIOS_REPO': scenarios_repo,
                'RALLY_SCENARIOS_BRANCH': scenarios_branch
            }
        ),
        Loader=yaml.SafeLoader
    )
    if kubectl_client.pods.present(name=pod_name, namespace=ns):
        LOG.warning("Pod '{0}' already exists. Deleting..".format(pod_name))
        kubectl_client.api_core.delete_namespaced_pod(pod_name, ns)

    pod = kubectl_client.pods.create(name=configmap_name, namespace=ns, body=pod_config)

    LOG.info("Wait Pod {}, timeout:{}s".format(configmap_name, timeout))
    pod.wait_phase('Running', timeout=timeout, interval=60)

    LOG.info("Copy artifacts")
    pod.cp_from_pod(source_dir='/var/rally/results')

    LOG.info("Done")
